//****************************************************************************
// @Module        Asynchronous Synchronous Serial Interface 0 (ASC0)
// @Filename      ASC0.c
// @Project       ASC_SSC_IO.dav
//----------------------------------------------------------------------------
// @Controller    Infineon TC1130
//
// @Compiler      Tasking
//
// @Codegenerator 1.4
//
// @Description   This file contains functions that use the ASC0 module.
//
//----------------------------------------------------------------------------
// @Date          2007-05-09 12:05:39
//
//****************************************************************************
// USER CODE BEGIN (ASC0_General,1)

// USER CODE END



//****************************************************************************
// @Project Includes
//****************************************************************************

#include "MAIN.h"

// USER CODE BEGIN (ASC0_General,2)

// USER CODE END


//****************************************************************************
// @Macros
//****************************************************************************

// USER CODE BEGIN (ASC0_General,3)

// USER CODE END


//****************************************************************************
// @Defines
//****************************************************************************

// USER CODE BEGIN (ASC0_General,4)

// USER CODE END


//****************************************************************************
// @Typedefs
//****************************************************************************

// USER CODE BEGIN (ASC0_General,5)

// USER CODE END


//****************************************************************************
// @Imported Global Variables
//****************************************************************************

// USER CODE BEGIN (ASC0_General,6)

// USER CODE END


//****************************************************************************
// @Global Variables
//****************************************************************************

// USER CODE BEGIN (ASC0_General,7)

// USER CODE END


//****************************************************************************
// @External Prototypes
//****************************************************************************

// USER CODE BEGIN (ASC0_General,8)

// USER CODE END


//****************************************************************************
// @Prototypes Of Local Functions
//****************************************************************************

// USER CODE BEGIN (ASC0_General,9)

// USER CODE END


//****************************************************************************
// @Function      void ASC0_vInit(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the initialization function of the ASC0 function 
//                library. It is assumed that the SFRs used by this library 
//                are in its reset state. 
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          2007-04-30
//
//****************************************************************************

// USER CODE BEGIN (Init,1)

// USER CODE END

void ASC0_vInit(void)
{
  // USER CODE BEGIN (Init,2)

  // USER CODE END

  ///  -----------------------------------------------------------------------
  ///  Configuration of the Module Clock:
  ///  -----------------------------------------------------------------------
  ///  - enable the ASC0 module
  ///  - clock divider for normal operation mode: System clock / 1 (= 75,0000 
  ///    MHz; 13,333 ns )
  ///  - the ASC0 module is stopped during sleep mode

  MAIN_vResetENDINIT();
  ASC0_CLC       =  0x00000100;  // load clock control register
  MAIN_vSetENDINIT();

  ///  -----------------------------------------------------------------------
  ///  Configuration of the ASC0 Baudrate Generator:
  ///  -----------------------------------------------------------------------
  ///  - the ASC0 module clock is 75 MHz
  ///  - additionally reduce serial clock to 2
  ///  - required baud rate = 115,200 kbaud
  ///  - real baud rate     = 117,188 kbaud
  ///  - deviation          = 1,725 %

  ASC0_BG        =  0x0013;      // load ASC0 baud rate time reload register

  ///  -----------------------------------------------------------------------
  ///  Configuration of the ASC0 Operation Mode:
  ///  -----------------------------------------------------------------------
  ///  - 8-bit data asychronous operation with one stop bit


  ///  - receiver is disabled

  ASC0_CON       =  0x0001;      // load ASC0 control register

  ///  -----------------------------------------------------------------------
  ///  FIFO Configuration:
  ///  -----------------------------------------------------------------------
  ///  - receive FIFO is disabled
  ///  - transmit FIFO is disabled

  ASC0_RXFCON    =  0x00000100;  // load ASC0 receive FIFO control register
  ASC0_TXFCON    =  0x00000100;  // load ASC0 transmit FIFO control register

  ///  -----------------------------------------------------------------------
  ///  Configuration of the used ASC0 Port Pins:
  ///  -----------------------------------------------------------------------
  ASC0_PISEL     =  0x0000;      // load peripheral input selection register

  ///  - P2.1 is used for ASC0 transmit output signal (ASC0_TXD)

  P2_ALTSEL0    |=  0x0002;      // select alternate output function
  P2_DIR        |=  0x0002;      // set direction register

  ///  -----------------------------------------------------------------------
  ///  Configuration of the used ASC0 Interrupts:
  ///  -----------------------------------------------------------------------
  ///  - Tx service request node configuration:
  ///  - Tx interrupt priority level (SRPN) = 3
  ///  - Tx CPU interrupt is selected

  ASC0_TSRC      =  0x00001003;  // set service request control register


  //   -----------------------------------------------------------------------
  //   Default Settings for Service Request Flags:
  //   -----------------------------------------------------------------------
  ASC0_TBSRC_SETR  =  1;         // indicates that the transmit buffer is 
                                 // empty

  // USER CODE BEGIN (ASC0_Function,3)

  // USER CODE END

  ASC0_CON      |=  0x8000;      // enable baud rate generator


} //  End of function ASC0_vInit


//****************************************************************************
// @Function      void ASC0_vSendData(ushort usData) 
//
//----------------------------------------------------------------------------
// @Description   This function writes a send data initialization word into 
//                the transmit buffer register.
//                
//                Note: 
//                In a multiprocessor system the master with this function 
//                has the possibility to send data to the selected slave. To 
//                achieve this, the 9th bit must set on zero.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    usData: 
//                Data to be send
//
//----------------------------------------------------------------------------
// @Date          2007-04-30
//
//****************************************************************************

// USER CODE BEGIN (SendData,1)

// USER CODE END

void ASC0_vSendData(ushort usData)
{
  ASC0_TBUF  = usData;   //  load transmit buffer register

} //  End of function ASC0_vSendData


//****************************************************************************
// @Function      void ASC0_viTx(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the transmit interrupt service routine for the 
//                ASC0. It is called when the sending of data is terminated. 
//                Please note that you have to add application specific code 
//                to this function.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          2007-04-30
//
//****************************************************************************

// USER CODE BEGIN (Tx,1)
static char *ss = "";						//Output Pointer
// USER CODE END

void __interrupt (ASC0_TINT) ASC0_viTx(void)
{

  // USER CODE BEGIN (Tx,2)
   	if(*ss)
		ASC0_vSendData(*ss++);				// send until string end
  // USER CODE END

} //  End of function ASC0_viTx




// USER CODE BEGIN (ASC0_General,10)

void ASC0_vWrite(char *s)
{
	while(*ss)								//wait if busy
	;
	ss = s;
	ASC0_TSRC_SETR = 1;						//Set interrupt request
}

// USER CODE END
