//****************************************************************************
// @Module        High-Speed Synchronous Serial Interface (SSC0)
// @Filename      SSC0.c
// @Project       ASC_SSC_IO.dav
//----------------------------------------------------------------------------
// @Controller    Infineon TC1130
//
// @Compiler      Tasking
//
// @Codegenerator 1.4
//
// @Description   This file contains functions that use the SSC0 module.
//
//----------------------------------------------------------------------------
// @Date          2007-05-09 12:05:39
//
//****************************************************************************

// USER CODE BEGIN (SSC0_General,1)

// USER CODE END



//****************************************************************************
// @Project Includes
//****************************************************************************

#include "MAIN.h"

// USER CODE BEGIN (SSC0_General,2)
#include<stdio.h>
// USER CODE END


//****************************************************************************
// @Macros
//****************************************************************************

// USER CODE BEGIN (SSC0_General,3)

// USER CODE END


//****************************************************************************
// @Defines
//****************************************************************************

// USER CODE BEGIN (SSC0_General,4)

// USER CODE END


//****************************************************************************
// @Typedefs
//****************************************************************************

// USER CODE BEGIN (SSC0_General,5)

// USER CODE END


//****************************************************************************
// @Imported Global Variables
//****************************************************************************

// USER CODE BEGIN (SSC0_General,6)

// USER CODE END


//****************************************************************************
// @Global Variables
//****************************************************************************

// USER CODE BEGIN (SSC0_General,7)

// USER CODE END


//****************************************************************************
// @External Prototypes
//****************************************************************************

// USER CODE BEGIN (SSC0_General,8)

// USER CODE END


//****************************************************************************
// @Prototypes Of Local Functions
//****************************************************************************

// USER CODE BEGIN (SSC0_General,9)

// USER CODE END


//****************************************************************************
// @Function      void SSC0_vInit(void) 
//
//----------------------------------------------------------------------------
// @Description   This is the initialization function of the SSC0 function 
//                library. It is assumed that the SFRs used by this library 
//                are in its reset state. 
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          2007-05-09
//
//****************************************************************************

// USER CODE BEGIN (Init,1)

// USER CODE END

void SSC0_vInit(void)
{

  volatile uword SSC0_CLC_read;

  // USER CODE BEGIN (Init,2)

  // USER CODE END

  ///  -----------------------------------------------------------------------
  ///  Configuration of the Module Clock:
  ///  -----------------------------------------------------------------------

  ///  - the SSC0 module is stopped during sleep mode
  ///  - normal divider mode is selected
  ///  - required SSC0 module clock is 75,000 MHz
  ///  - real SSC0 module clock is 75,000 MHz

  MAIN_vResetENDINIT();
  SSC0_CLC       =  0x00000000;  // load clock control register
  SSC0_CLC_read  =  SSC0_CLC;    // Read clock control register
  SSC0_FDR       =  0x000043FF;  // load fractional divider register
  MAIN_vSetENDINIT();

  ///  -----------------------------------------------------------------------
  ///  Configuration of the SSC0 Baud Rate Generator:
  ///  -----------------------------------------------------------------------
  ///  - required baud rate = 400,200 kbaud
  ///  - real baud rate     = 400,031 kbaud	  ???
  ///  - deviation          = -0,147 %

  SSC0_BR        =  0x005d;      // load SSC0 baud rate time reload register

  ///  -----------------------------------------------------------------------
  ///  Configuration of the SSC0 Operation Mode:
  ///  -----------------------------------------------------------------------
  ///  - this device is configured as SSC0 master
  ///  - transfer data width is 8 bit
  ///  - transfer/receive MSB first
  ///  - shift transmit data on the leading clock edge, latch on trailing edge
  ///  - idle clock line is low, leading clock edge is low-to-high transition
  ///  - ignore receive error
  ///  - ignore phase error

  SSC0_CON       =  0x4017;      // load SSC0 control register

  ///  -----------------------------------------------------------------------
  ///  FIFO Configuration:
  ///  -----------------------------------------------------------------------
  ///  - receive FIFO is disabled
  ///  - transmit FIFO is disabled

  SSC0_RXFCON    =  0x00000102;  // load SSC0 receive FIFO control register
  SSC0_TXFCON    =  0x00000102;  // load SSC0 transmit FIFO control register

  ///  -----------------------------------------------------------------------
  ///  Configuration of the SSC0 slave select signals:
  ///  -----------------------------------------------------------------------

  ///  SLSO3:
  ///  - SLSO3 output is enabled
  ///  - SLSO3 line is at low level during the chip select active time
  ///  - P2.12 is used for SSC0 Slave Select output signal 3(SSC0_SLSO)

  P2_ALTSEL1    |=  0x1000;      // select alternate output function
  P2_DIR        |=  0x1000;      // set direction register
  ///  SLSOx timing:
  ///  - slave output select leading delay: 0 SCLK periods
  ///  - slave output select trailing delay: 0 SCLK periods
  ///  - slave output select inactive delay: 0 SCLK periods

  SSC0_SSOC      =  0x00000800;  // load SSC0 slave select output control 
                                 // register
  SSC0_SSOTC     =  0x00000000;  // load SSC0 slave select output timing 
                                 // control register

  ///  -----------------------------------------------------------------------
  ///  Configuration of the used SSC0 Port Pins:
  ///  -----------------------------------------------------------------------
  SSC0_PISEL     =  0x0000;      // load peripheral input selection register

  ///  - P2.4 is used for SSC0 clock output signal (SSC0_SCLK)
  ///  - P2.3 is used for SSC0 master transmit signal (SSC0_MTSR)
  ///  - P2.2 is used for SSC0 master receive signal (SSC0_MRST)

  P2_ALTSEL0    |=  0x0018;      // select alternate output function
  P2_DIR        |=  0x0018;      // set direction register

  ///  -----------------------------------------------------------------------
  ///  Configuration of the used SSC0 Interrupts:
  ///  -----------------------------------------------------------------------
  ///  - transmit service request node configuration:
  ///  - transmit interrupt priority level (SRPN) = 8
  ///  - transmit CPU interrupt is selected

  SSC0_TSRC      =  0x00001008;  // set service request control register

  ///  - receive service request node configuration:
  ///  - receive interrupt priority level (SRPN) = 9
  ///  - receive CPU interrupt is selected

  SSC0_RSRC      =  0x00001009;  // set service request control register


  // USER CODE BEGIN (Init,3)

  // USER CODE END

  SSC0_CON      |=  0x8000;      // enable transmission and reception

} //  End of function SSC0_vInit


//****************************************************************************
// @Function      void SSC0_vSendData(ushort usData) 
//
//----------------------------------------------------------------------------
// @Description   The master device can initiate the first data transfer by 
//                writing the transmit data into transmit buffer. This value 
//                is copied into the shift register (which is assumed to be 
//                empty at this time), and the selected first bit of the 
//                transmit data is placed onto the MTSR line on the next 
//                clock from the baud rate generator. 
//                A slave device immediately outputs the selected first bit 
//                (MSB or LSB of the transfer data) at pin MRST, when the 
//                contents of the transmit buffer are copied into the slave's 
//                shift register. 
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    usData: 
//                Data to be send
//
//----------------------------------------------------------------------------
// @Date          2007-05-09
//
//****************************************************************************

// USER CODE BEGIN (SendData,1)

// USER CODE END

void SSC0_vSendData(ubyte usData)
{
  SSC0_TB  = usData;   							// load transmit buffer register
  while((SSC0_STAT & SSC0_STAT_BSY) == 1)		//solange Sendevorgang luft...warten
  {
  	printf(".");
  }												
}
  //  End of function SSC0_vSendData


//****************************************************************************
// @Function      ushort SSC0_usGetData(void) 
//
//----------------------------------------------------------------------------
// @Description   This function returns the contents of the receive buffer. 
//                When the receive interrupt request flag is set this implies 
//                that data is available for reading in the receive buffer. 
//
//----------------------------------------------------------------------------
// @Returnvalue   Received data
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          2007-05-09
//
//****************************************************************************

// USER CODE BEGIN (GetData,1)

// USER CODE END

ubyte SSC0_usGetData(void)
{
  return(SSC0_RB);     							// return receive buffer register

} //  End of function SSC0_usGetData


//****************************************************************************
// @Function      void SSC0_viTx(void) 
//
//----------------------------------------------------------------------------
// @Description   This interrupt service routine is called when the contents 
//                of the transmit buffer have been loaded into the shift 
//                register. This means that the transmit buffer can now be 
//                loaded with the next data. 
//                Please note that you have to add application specific code 
//                to this function.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          2007-05-09
//
//****************************************************************************

// USER CODE BEGIN (Tx,1)
static ubyte *ss = "";						//Output Pointer
// USER CODE END

void __interrupt (SSC0TINT) SSC0_viTx(void)
{
  // USER CODE BEGIN (Tx,4)
 	if(*ss)
	{
		SSC0_vSendData(*ss++);				//noch CS auf negatives Potential legen!!
	}

  // USER CODE END

} //  End of function SSC0_viTx


//****************************************************************************
// @Function      void SSC0_viRx(void) 
//
//----------------------------------------------------------------------------
// @Description   This interrupt service routine is called when the contents 
//                of the shift register have been loaded into the receive 
//                buffer. The receive buffer must be read before receiving 
//                the next data. If the receive buffer is not read before 
//                receiving new data, the receive error flag is set. 
//                Please note that you have to add application specific code 
//                to this function.
//
//----------------------------------------------------------------------------
// @Returnvalue   None
//
//----------------------------------------------------------------------------
// @Parameters    None
//
//----------------------------------------------------------------------------
// @Date          2007-05-09
//
//****************************************************************************

// USER CODE BEGIN (Rx,1)

// USER CODE END

void __interrupt (SSC0RINT) SSC0_viRx(void)
{
  // USER CODE BEGIN (Rx,4)				 
	
  // USER CODE END

} //  End of function SSC0_viRx

//************************************************************************************************************************************************

// USER CODE BEGIN (SSC0_General,10)
void SSC0_vWrite(ubyte *s)
{
	while(*ss)												//wait if busy
	;
	ss = s;
	SSC0_TSRC      =  0x00009008;							//Set interrupt request
}
//************************************************************************************************************************************************
ubyte SSC0_Read(void)
{
	ubyte Byte=0;

	SSC0_vSendData(0xFF);
	Byte=SSC0_usGetData();
	return (Byte);
}
//************************************************************************************************************************************************
ubyte sd_write_command(ubyte *cmd)
{	
	ubyte tmp = 0xff;
	ubyte a=0;
  							    
	int Timeout = 0;

 	SD_Disable();											//SD-Karten CS auf HIGH Pegel
 	SSC0_vSendData(0xff);									//Sendet 8 Takt/Clock Impulse
	SD_Enable();											//SD-Karten CS auf LOW Pegel
   	
   	while(a<6)												//Sendet 6 Byte Commando zur SD-Karte
	{
		printf("Commando %x\n",*cmd);
	    SSC0_vSendData(*cmd++);		
	    a++;
	}
   
	while(tmp == 0xff)
	{
		tmp = SSC0_Read();
		printf("tmp= %x \n",tmp);
		if(Timeout++ >10)
			break;										    //Abbruch da die SD-Karte nicht Antwortet
	} 
   	
	return (tmp);
}
//************************************************************************************************************************************************
ubyte SD_Init(void)
{
	ubyte Timeout = 0;
	ubyte var=0;
	
	for(ubyte a=0;a<200;a++)
		__nop();											//kurze Wartezeit


	SD_Disable();											//CS der SD-Karte auf High Pegel

  	printf("\nInitialisiere SD-Karte in den SPI-Mode\n");
	for(ubyte i=0;i<10;i++)			   						//Initialisiere SD-Karte in den SPI-Modus
	{
	  	SSC0_TB = 0xff;									    //Sendet min 74+ Takte an die SD-Karte / Pseudodaten in den Transmitbuffer
		printf("Takte gesendet: %d\n",(i+1)*8);
	}
   	
   	printf("\nSenden von Commando 0\n");   					//Commando 0 => resets the SD-Memory Card...GO_IDLE_STATE..Response 0x01
	ubyte cmd[] = {0x40,0x00,0x00,0x00,0x00,0x95};			
	while(sd_write_command(cmd)!=0x01)						//nur bei rckgabewert 0x01 ist die das Reset der sd-karte fehlerfrei
	{
		if(Timeout++ >= 10)
		{
			return (1);
		}
	}
   		
   	printf("Senden von Commando 1\n"); 						//Commando 1 => activates the cards initialization process...SEND_OP_COND....Response 0x00
    cmd[0] = 0x41;
	cmd[5] = 0xff;
	while(sd_write_command(cmd)==0x00)
	{
		if(Timeout++ >= 10)
		{
			return (1);
		}
	}

	SD_Disable();											//CS der SD-Karte auf HIGH Pegel
	return (0);
}
// USER CODE END

